/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 *
 *******************************************************************************
 *
 * $Id$
 *
 ******************************************************************************/
package it.tidalwave.semantic.node;

import javax.annotation.Nonnull;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collection;
import java.io.IOException;
import java.awt.event.ActionEvent;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import javax.swing.AbstractAction;
import javax.swing.Action;
import org.openide.util.datatransfer.PasteType;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.nodes.NodeAdapter;
import org.openide.nodes.NodeListener;
import org.openide.nodes.NodeMemberEvent;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.openide.explorer.ExplorerPanel;
import it.tidalwave.openide.explorer.view.EnhancedBeanTreeView;
import it.tidalwave.semantic.other.Entity;
import it.tidalwave.semantic.node.impl.EntityNode;

/*******************************************************************************
 *
 * The TagDropArea is a visual component where tags can be dropped. It provides
 * a <code>getTags()</code> method to retrieve all the contained tags.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class EntityDropArea extends ExplorerPanel
  {
    private static final String CLASS = EntityDropArea.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    
    private final Node containerNode = new EntityContainerNode();
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    private final Action removeSelectedTagsAction = new AbstractAction()
      {
        public void actionPerformed (@Nonnull final ActionEvent event)
          {
            final Node[] nodes = getExplorerManager().getSelectedNodes();
            containerNode.getChildren().remove(nodes);
          }
      };
      
    /***************************************************************************
     *
     *
     **************************************************************************/
    private final Action removeAllTagsAction = new AbstractAction()
      {
        public void actionPerformed (@Nonnull final ActionEvent event)
          {
            final Node[] all = containerNode.getChildren().getNodes();
            containerNode.getChildren().remove(all);
          }
      };
      
    /***************************************************************************
     *
     *
     **************************************************************************/
    private final NodeListener nodeListener = new NodeAdapter()
      {
        @Override
        public void childrenAdded (@Nonnull final NodeMemberEvent event)
          {
            update();
          }

        @Override
        public void childrenRemoved (@Nonnull final NodeMemberEvent event)
          {
            update();
          }
        
        private void update()
          {
            removeAllTagsAction.setEnabled(containerNode.getChildren().getNodesCount() > 0);  
          }
      };
      
    /***************************************************************************
     *
     *
     **************************************************************************/
    private final PropertyChangeListener explorerManagerListener = new PropertyChangeListener()
      {
        public void propertyChange (@Nonnull final PropertyChangeEvent event)
          {
            removeSelectedTagsAction.setEnabled(getExplorerManager().getSelectedNodes().length > 0);
          }
      };
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    class EntityContainerNode extends AbstractNode
      {
        public EntityContainerNode()
          {
            super(new Children.Array()); // SortedArray    
            setName("Entities");
          }

        public boolean canPaste()
          {
            return true;
          }

        @Override
        public PasteType getDropType (@Nonnull final Transferable transferable,
                                      final int action,
                                      final int index)
          {
            try  
              {
                if (transferable.isDataFlavorSupported(EntityNode.ENTITY_NODE_FLAVOR))
                  {
                    final Node tagNode = (Node)transferable.getTransferData(EntityNode.ENTITY_NODE_FLAVOR);

                    return new PasteType()
                      {
                        public Transferable paste() 
                          throws IOException 
                          {
                            getChildren().add(new Node[]{tagNode.cloneNode()});
                            return null; // ExTransferable.EMPTY?
                          }
                      };
                  }
              } 
            catch (IOException e) 
              {
                logger.throwing(CLASS, "getDropType()", e);
              } 
            catch (UnsupportedFlavorException e) 
              {
                logger.throwing(CLASS, "getDropType()", e);
              }
            return null;
          }
      };

    /***************************************************************************
     *
     *
     **************************************************************************/
    public EntityDropArea()
      {
        super(new EnhancedBeanTreeView()); 
//        super(new EnhancedListView()); // throws exception when dropping
        ((EnhancedBeanTreeView)getView()).setRootVisible(false);
        getExplorerManager().setRootContext(containerNode);
        containerNode.addNodeListener(nodeListener);
        getExplorerManager().addPropertyChangeListener(explorerManagerListener);
        removeAllTagsAction.setEnabled(false);
        removeSelectedTagsAction.setEnabled(false);
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public Collection<Entity> getEntities()
      {
        final Collection<Entity> result = new ArrayList<Entity>();
        final Node node = getExplorerManager().getRootContext();
        
        for (final Node childNode : node.getChildren().getNodes())
          {
            final Entity entity = childNode.getLookup().lookup(Entity.class);
            
            if (entity != null)
              {
                result.add(entity);
              }
          }
        
        return result;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public Action getRemoveSelectedNodesAction()
      {
        return removeSelectedTagsAction;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public Action getRemoveAllNodesAction()
      {
        return removeAllTagsAction;
      }
  } 
