/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 *
 *******************************************************************************
 *
 * $Id$
 *
 ******************************************************************************/
package it.tidalwave.semantic.node.impl;

import javax.annotation.Nonnull;
import java.util.logging.Logger;
import java.util.ArrayList;
import java.util.List;
import java.util.Collections;
import java.util.Comparator;
import java.text.Collator;
import java.io.IOException;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import org.openide.util.datatransfer.PasteType;
import org.openide.loaders.DataObject;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.nodes.ChildFactory;
import it.tidalwave.util.Id;
import it.tidalwave.netbeans.draganddrop.DataObjectPasteType;
import it.tidalwave.netbeans.draganddrop.DataObjectTransferable;
import it.tidalwave.netbeans.draganddrop.TransferableSupport;
import it.tidalwave.openide.nodemanager.NodeDelegateFactory;
import it.tidalwave.semantic.other.Entity;
import it.tidalwave.semantic.other.Relation;
import it.tidalwave.semantic.other.Type;
import it.tidalwave.semantic.node.dnd.PasteTypeFactory;

/*******************************************************************************
 *
 * A Node subclass for representing a Tag.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class EntityNode extends SemanticNode
  {
    private static final String CLASS = EntityNode.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    
    /** This DataFlavor maps to the tag contained by this node. */
    public static final DataFlavor ENTITY_FLAVOR = new DataFlavor(Entity.class, "Entity");
    
    /** This DataFlavor maps to the tag node itself. */
    public static final DataFlavor ENTITY_NODE_FLAVOR = new DataFlavor(EntityNode.class, "EntityNode");
    
    /** The bound tag. */
    private final Entity entity;

    private final PasteTypeFactory pasteTypeFactory;
    
    protected final EntityChildFactory entityChildFactory;
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    protected static class EntityChildFactory extends ChildFactory<Entity>
      {
        @Nonnull
        private final Entity entity;

        public EntityChildFactory (@Nonnull final Entity entity)
          {
            this.entity = entity;
          }
        
        protected boolean createKeys (@Nonnull final List<Entity> keys)
          {
            final List<Entity> entities = new ArrayList<Entity>();
            
            for (final Relation relation : entity.findRelations()) // FIXME: put a filter
              {
                if ((relation.getObject() == entity) && relation.getType().stringValue().contains("partOf")) // FIXME
                  {
                    entities.add((Entity)relation.getSubject());
                  }
              }
            
            final Collator collator = Collator.getInstance();

            Collections.sort(entities, new Comparator<Entity>()
              {
                public int compare (final Entity t1, final Entity t2)
                  {
                    return collator.compare(t1.getDisplayName(), t2.getDisplayName());
                  }
              });

            keys.addAll(entities);
            return true;
          }
        
        @Override
        @Nonnull
        protected final Node createNodeForKey (@Nonnull final Entity entity)
          {
            final NodeDelegateFactory nodeFactory = entity.getLookup().lookup(NodeDelegateFactory.class);
            assert nodeFactory != null : "No NodeDelegateFactory for " + entity;
            return nodeFactory.findOrCreateNodeDelegate();
          }

        public final void refresh()
          {
            refresh(false);
          }
      };
            
    /***************************************************************************
     *
     * Creates a new instance of Node bound to the given tag.
     *
     * @param  tag  the tag
     *
     **************************************************************************/
    /* package */ EntityNode (@Nonnull final Entity entity)
      {
        this(entity, new EntityChildFactory(entity));
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    private EntityNode (@Nonnull final Entity entity,
                        @Nonnull final EntityChildFactory entityChildFactory)
      {
        super(Children.create(entityChildFactory, true), entity);
        this.entity = entity;
        this.entityChildFactory = entityChildFactory;
        pasteTypeFactory = new PasteTypeFactory(entity);
//        refreshAttributes();
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    protected EntityNode (@Nonnull final EntityChildFactory entityChildFactory)
      {
        super(Children.create(entityChildFactory, true));
        this.entity = null;
        this.entityChildFactory = entityChildFactory;
        pasteTypeFactory = null;
//        refreshAttributes();
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    public void destroy() 
      throws IOException
      {
//       release listeners
        super.destroy();
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    @Nonnull
    public String getDisplayName()
      {
        return findDisplayName(entity);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    @Nonnull
    public String getName()
      {
        return findDisplayName(entity);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public final void refreshChildren()
      {
        entityChildFactory.refresh();
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    @Nonnull 
    public Transferable clipboardCopy() 
      throws IOException 
      {
        return new EntityTransferable();
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    @Nonnull
    public String toString()
      {
        return "EntityNode[" + entity + "]";
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    protected void createPasteTypes (@Nonnull final Transferable transferable,
                                     @Nonnull final List<PasteType> pasteTypes)
      {
        if (pasteTypeFactory != null)
          {
            pasteTypeFactory.createPasteTypes(transferable, pasteTypes);
          }
        
        super.createPasteTypes(transferable, pasteTypes);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    private String findDisplayName (@Nonnull final Entity entity)
      {
        final String displayName = entity.getDisplayName();

        if (entity.getId().stringValue().equals(displayName))
          {
            // FIXME: make it parametrized
            // FIXME: don't know whether this should go to Entity, with the
            // candidate properties registered in the Repository.
            // Otherwise you could add a statement that declares the relevant
            // properties as subPropertyOf rdfs:label.
            // But you can't manage more complex scenarios where you need to
            // recover the displayName from a relation. Unless owl:functionalProperty does it?
            final String x = "http://www.daml.org/2001/01/gedcom/gedcom#name";
            final Type t = entity.getRepository().findOrCreateType(new Id(x), "Name");
            final Object property = entity.getProperty(t);

            if (property != null)
              {
                return property.toString();
              }
          }

        return displayName;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    class EntityTransferable extends TransferableSupport
      {
        public EntityTransferable()
          {
            addObject(ENTITY_FLAVOR, entity);
            addObject(ENTITY_NODE_FLAVOR, EntityNode.this);
//            addObject(DataFlavor.stringFlavor, entity.getQualifiedName());
            addObject(DataObjectTransferable.DATA_OBJECT_FLAVOR, new EntityPasteAction());
          }
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    // FIXME: this requires a dependency on Thumbnail
    class EntityPasteAction implements DataObjectPasteType.Action // FIXME: try to merge / compose with PhotoDataObjectPasteType
      {
        public void paste (@Nonnull final DataObject dataObject)
          {
//            try
//              {
//                CatalogLocator.findCatalog().findTagSet(dataObject).add(tag);
//              }
//            catch (CatalogException e)
//              {
//                logger.throwing(CLASS, "TagPasteAction.paste()", e); // FIXME: rethrow
//              }
          }
      }             
  }
